# $Id: Configuration.pm 1642 2009-04-19 15:47:43Z wsl $
# $URL: http://svn.zo.oi/wsl/src/libxyzzy-perl/lib/Xyzzy/Configuration.pm $

use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Configuration;

use Spiffy -Base;

use Apache2::CmdParms ();
use Apache2::Module ();
use Apache2::Directive ();
use Apache2::ServerUtil;

field 'prefix';
field 'templatedir';
field 'staticrootdir';

Apache2::Module::add(__PACKAGE__, [
	{ name => 'LocationPrefix' },
	{ name => 'TemplateDir' },
	{ name => 'StaticRootDir' }
]);

sub LocationPrefix {
	my $prefix = $_[1];
	$prefix =~ "/$prefix/";
	$prefix =~ s,/+,/,g;
	$self->prefix($prefix);
}
sub TemplateDir {
	my $dir = $_[1];
	stat $dir;
	warn "TemplateDir $dir doesn't exist\n" unless -e _;
	warn "TemplateDir $dir is not a directory\n" unless -d _;
	$self->templatedir($dir);
}
sub StaticRootDir {
	my $dir = $_[1];
	stat $dir;
	warn "StaticRootDir $dir doesn't exist\n" unless -e _;
	warn "StaticRootDir $dir is not a directory\n" unless -d _;
	$self->staticrootdir($dir);
}

sub SERVER_MERGE() { bless {map {%$_} @_}, ref $_[0] }
sub DIR_MERGE() { bless {map {%$_} @_}, ref $_[0] }

sub get() {
	my $class = shift;
	my $rec = shift;
	my @chain = reverse @{$class->isachain};
	my $config = $class->new;
	my $server = $rec->server;
	my $pdc = $rec->per_dir_config;
	foreach my $c (@chain) {
		my $cfg = Apache2::Module::get_config($c, $server, $pdc);
		@$config{keys %$cfg} = values %$cfg
			if defined $cfg;
	}
	return $config;
}

our %isacache;

sub isachain {
	no strict 'refs';
	my $class = ref $self || $self;
	return $isacache{$class}
		if exists $isacache{$class};
	my @chain;
	my $todo = {$class => undef};
	my %done;
	while(%$todo) {
		my %next;
		while(my ($c) = each(%$todo)) {
			push @chain, $c;
			undef $done{$c};
			@next{@{"$c\::ISA"}} = ()
		}
		delete @next{keys %done};
		$todo = \%next;
	}
	$isacache{$class} = \@chain;
	return \@chain;
}
